struct VertexToPixel
{
    float4 Position     : POSITION;
    float2 TexCoords    : TEXCOORD0;
};


struct PixelToFrame
{
    float4 Color : COLOR0;
};

static const float gauss[4] = {
	0.6193, 0.0838,
	0.0838, 0.0113
};

Texture xImageTarget : ImageMap;
float2 xViewportSize: RenderTargetDimensions;

//sampler ImageSampler = sampler_state { texture = <xImageTarget> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};
sampler ImageSampler = sampler_state { texture = <xImageTarget> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};

VertexToPixel ScreenQuadVertexShader( float4 inPos : POSITION, float4 inTexCoord : TEXCOORD0)
{
    VertexToPixel Output = (VertexToPixel)0;

    Output.Position = inPos;
    Output.TexCoords = inTexCoord;

    return Output;
}

PixelToFrame BlurRenderPixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    
    float4 color = float4(0.0,0.0,0.0,1.0);
    for(int x = -1 ; x < 2; x++)
    {
    	for (int y = -1; y < 2; y++)
    	{
    		float2 point = float2(PSIn.TexCoords.x + x / xViewportSize.x, PSIn.TexCoords.y + y / xViewportSize.y);
    		//float fact = pow(1.1,-0.5 * y * y)/5;
    		float fact = gauss[abs(x) + 2 * abs(y)];
    		float4 sample = tex2D(ImageSampler, point);
    		color.rgb += sample.rgb * fact;
    	}
    }
    
    float4 ColorImage = tex2D(ImageSampler, PSIn.TexCoords);

    Output.Color = color;
    return Output;
}

technique Blur
{
    pass BlurRenderPass
    {
    	VertexShader = compile vs_1_1 ScreenQuadVertexShader();
    	Pixelshader = compile ps_3_0 BlurRenderPixelShader();
    }
}