struct VertexToPixel
{
    float4 Position     : POSITION;
    float3 Normal		: NORMAL;
    float2 TexCoords    : TEXCOORD0;
    float4 Tangent		: TANGENT;
    float4 Binormal		: BINORMAL;

    
    float3 WorldVertPos			: TEXCOORD1;
    float3 WorldViewPos			: TEXCOORD2;
    float3 ViewVec			: TEXCOORD3;
};

struct PixelToFrame
{
    float4 Color0 : COLOR0;
    float4 Color1 : COLOR1;
};

struct Light
{
	float3 Pos;
	float4 DiffCol;
    float Diffuse;
    float Ambient;
    float Radius;
};



Light xLights[4] : LightArray;
Light xShadowmapLights[2] : ShadowMapLightArray;

Texture xShadowmap0 : Shadowmap0;
Texture xShadowmap1 : Shadowmap1;

int xNumLight : NumLights;
int xNumShadowMaplight : NumShadowmap;

Texture xDiffuseTexture : DiffuseMap;
Texture xSurfaceTexture : SurfaceMap;
Texture xNormalTexture : NormalMap;
float4x4 xView : View;
float4x4 xProjection : Projection;
float4x4 xWorld : World;
float4 xViewPos : ViewPos;
float4x4 xWorldViewProj : WorldViewProjection;
float4x4 xViewInverse : ViewInverse;

sampler DiffuseSampler = sampler_state { texture = <xDiffuseTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = WRAP; AddressV = WRAP;};
sampler SurfaceSampler = sampler_state { texture = <xSurfaceTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = WRAP; AddressV = WRAP;};
sampler NormalSampler = sampler_state { texture = <xNormalTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = WRAP; AddressV = WRAP;};

samplerCUBE ShadowmapSampler[2] = {
	sampler_state { texture = <xShadowmap0> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE;},
	sampler_state { texture = <xShadowmap1> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE;}
	};





VertexToPixel TextureVertexShader( float4 inPos : POSITION, float3 normal : NORMAL, float3 tangent : TANGENT, float4 inTexCoord : TEXCOORD0)
{
    VertexToPixel Output = (VertexToPixel)0;

	Output.TexCoords = inTexCoord;

	Output.Position = mul(inPos,xWorldViewProj);
	
	float4x4 WorldRotation = xWorld;
	WorldRotation._41_42_43 = 0.0;
    Output.Normal = mul(normal, WorldRotation);
	Output.Tangent = mul(tangent, WorldRotation);
	Output.Binormal = mul(cross(normal, tangent), WorldRotation);
	
	Output.WorldViewPos.xyz =  - mul(xView, float4(0,0,10,1));
	
	Output.WorldVertPos = mul(float4(inPos.xyz, 1.0), xWorld).xyz;
	
	Output.ViewVec = xViewPos - Output.WorldVertPos;
    return Output;
}

PixelToFrame TexturePixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    
    float3 ViewVec = normalize(PSIn.ViewVec);
    
    //float height = (0.5 - tex2D(NormalSampler, PSIn.TexCoords).a) * 0.4;
	//float2 Lookuppos = ViewVec.xy * float2(-1.0, 1.0) * height + PSIn.TexCoords;
    float2 Lookuppos = PSIn.TexCoords;
    
    float3 normal = normalize(PSIn.Normal);
    
    float3 tangent = normalize(PSIn.Tangent);
    
    float3 binormal = normalize(PSIn.Binormal);
    
    float3 normalmap = normalize(tex2D(NormalSampler,Lookuppos) * 2.0 - float3(1,1,1));
    
    float3 fnormal = tangent * normalmap.x;
    fnormal -= binormal * normalmap.y;
    fnormal += normal * normalmap.z;
    fnormal = normalize(fnormal);
	
    float3 Diffuse = 0.0;
    float Specular = 0.0;
    float Ambient = 0.0;
    
    for (int i = 0; i <  xNumLight && i < 4; i++)
    {
    	float3 LightVec = normalize(xLights[i].Pos - PSIn.WorldVertPos);
    	float falloff = (xLights[i].Radius - length(xLights[i].Pos - PSIn.WorldVertPos)) / xLights[i].Radius;
  		falloff = clamp(falloff,0,1);
  		
    	Diffuse += dot(fnormal, LightVec) * xLights[i].Diffuse * xLights[i].DiffCol * falloff;
		Ambient += xLights[i].Ambient;
    	float3 halfway = normalize(LightVec + ViewVec);    
    	Specular += pow(saturate(dot(fnormal, halfway)),120) * xLights[i].Diffuse * falloff;
    }
    
    for (int i = 0; i <  xNumShadowMaplight && i < 2; i++)
    {
    	float3 LightVec = xShadowmapLights[i].Pos - PSIn.WorldVertPos;
    	float3 lookpos = PSIn.WorldVertPos - xShadowmapLights[i].Pos;
    	float4 shadowdepth = texCUBE(ShadowmapSampler[i], normalize(lookpos));
		float intensity = length(LightVec) > shadowdepth.r ? 0.0 : 1.0;
		//s = x != 0.0 ? Math.Sin(x)/x : 1.0;

		
    	LightVec = normalize(LightVec);
    	float falloff = (xShadowmapLights[i].Radius - length(xShadowmapLights[i].Pos - PSIn.WorldVertPos)) / xShadowmapLights[i].Radius;
  		falloff = clamp(falloff,0,1);
    	Diffuse += (dot(fnormal, LightVec) * xShadowmapLights[i].Diffuse * xShadowmapLights[i].DiffCol * falloff) * intensity;
		Ambient += xShadowmapLights[i].Ambient;
    	float3 halfway = normalize(LightVec + ViewVec);    
    	Specular += (pow(saturate(dot(fnormal, halfway)),120) * xShadowmapLights[i].Diffuse * falloff) * intensity;
		
    }
    
    Ambient = saturate(Ambient);
    Diffuse = saturate(Diffuse);
    Specular = saturate(Specular);

	float3 diffcol = tex2D(DiffuseSampler, Lookuppos);
	float3 surfacecol = tex2D(SurfaceSampler, Lookuppos);
    Output.Color0.rgb = diffcol * Ambient + diffcol * Diffuse + surfacecol.r * Specular;
 	Output.Color0.w = 1.0;
    Output.Color1.rgb = 1.0;
    Output.Color1.w = 1.0;

    return Output;
}

technique Normal
{

    pass Pass0
    {        
        VertexShader = compile vs_3_0 TextureVertexShader();
        PixelShader = compile ps_3_0 TexturePixelShader();
    }
}