struct VertexToPixel
{
    float4 Position     : POSITION;
    float2 TexCoords    : TEXCOORD0;
};


struct PixelToFrame
{
    float4 Color : COLOR0;
};


Texture xImageTarget : ImageMap;
Texture xGlowTarget : GlowMap;
Texture xGlowHor : GlowHorMap;
Texture xGlowVer : GlowVerMap;
float2 xViewportSize: RenderTargetDimensions;

static const float gauss[15] = {
	0.0174813,
	0.035994,
	0.0663181,
	0.10934,
	0.161314,
	0.212965,
	0.251589,
	0.265962,
	0.251589,
	0.212965,
	0.161314,
	0.10934,
	0.0663181,
	0.035994,
	0.0174813
};

sampler ImageSampler = sampler_state { texture = <xImageTarget> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};
sampler GlowSampler = sampler_state { texture = <xGlowTarget> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};
sampler GlowHorSampler = sampler_state { texture = <xGlowHor> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};
sampler GlowVerSampler = sampler_state { texture = <xGlowVer> ; magfilter = POINT; minfilter = POINT; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};

VertexToPixel ScreenQuadVertexShader( float4 inPos : POSITION, float4 inTexCoord : TEXCOORD0)
{
    VertexToPixel Output = (VertexToPixel)0;

    Output.Position = inPos;
    Output.TexCoords = inTexCoord;

    return Output;
}

PixelToFrame GlowVertPixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    
    float4 Glow = float4(0.0,0.0,0.0,0.0);
    for (int y = -7; y < 8; y++)
    {
    	float2 point = float2(PSIn.TexCoords.x, PSIn.TexCoords.y + y / xViewportSize.y);
    	//float fact = pow(1.1,-0.5 * y * y)/5;
    	float fact = gauss[y+7];
    	float4 sample = tex2D(GlowSampler, point);
    	Glow.rgb += sample.rgb * sample.w * fact;
    	Glow.w += sample.w * fact;
    }
    Output.Color = Glow;  
    return Output;
}

PixelToFrame GlowHorPixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    
    float4 Glow = float4(0.0,0.0,0.0,0.0);
    for (int x = -7; x < 8; x++)
    {
    	float2 point = float2(PSIn.TexCoords.x + x  / xViewportSize.x, PSIn.TexCoords.y);
    	//float fact = pow(1.1,- 0.5 * x * x)/5;
    	float fact = gauss[x+7];
    	float4 sample = tex2D(GlowVerSampler, point);
    	Glow.rgb += sample.rgb * sample.w * fact;
    	Glow.w += sample.w * fact;
    }
    Output.Color = Glow;  
    return Output;
}

PixelToFrame GlowRenderPixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    
    float4 ColorImage = tex2D(ImageSampler, PSIn.TexCoords);
    float4 Glow = tex2D(GlowHorSampler, PSIn.TexCoords);
    
    Output.Color.rgb = ColorImage * (1.0 - Glow.w) + Glow.rgb * Glow.w;
    Output.Color.w = 1.0;
    
    //Output.Color = Glow;
    return Output;
}

technique Glow
{
    pass GlowVertPass
    {        
        VertexShader = compile vs_1_1 ScreenQuadVertexShader();
        PixelShader = compile ps_3_0 GlowVertPixelShader();
    }
    pass GlowHorPass
    {        
        VertexShader = compile vs_1_1 ScreenQuadVertexShader();
        PixelShader = compile ps_3_0 GlowHorPixelShader();
    }
    pass GlowRenderPass
    {
    	VertexShader = compile vs_1_1 ScreenQuadVertexShader();
    	Pixelshader = compile ps_3_0 GlowRenderPixelShader();
    }
}